
library(deSolve)
set.seed(1208)

# Model functions ---------------------------------------------------------

#analytical equilibria
REeq <- function(pars) {
  with(as.list(pars), {
    (m+e)/r
  })
}

RCeq <- function(pars) {
  with(as.list(pars), {
    m/r
  })
}

NE_mono_eq <- function(pars) {
  with(as.list(pars), {
    #a/2*(e*S0*g*qz - e*mz*q - m*mz*q)
    #(S0*a*e*g*qz+mz*((-a*m*q)-a*e*q))/((e*g*m*q+e^2*g*q)*qz)
    (S0*a*e*g*qz+((-a*m)-a*e)*mz*q)/((e*g*m+e^2*g)*q*qz)
  })
}

NEeq <- function(pars) {
  with(as.list(pars), {
    c((sqrt((S0^2*a^2*e^2*g^2*qz^2+((-2*S0*a^2*e*g*m)-2*S0*a^2*e^2*g)*mz*q*qz
             +(a^2*m^2+2*a^2*e*m+a^2*e^2)*mz^2*q^2)
            *r^2
            +(((-2*a*d*e^2*g*m)-2*a*d*e^3*g)*mz*q*qz-2*S0*a*d*e^3*g^2*qz^2)
            *r+d^2*e^4*g^2*qz^2)
       +(S0*a*e*g*qz+((-a*m)-a*e)*mz*q)*r-d*e^2*g*qz)
      /((2*e*g*m+2*e^2*g)*q*qz*r),
      -(sqrt((S0^2*a^2*e^2*g^2*qz^2+((-2*S0*a^2*e*g*m)-2*S0*a^2*e^2*g)
              *mz*q*qz
              +(a^2*m^2+2*a^2*e*m+a^2*e^2)*mz^2*q^2)
             *r^2
             +(((-2*a*d*e^2*g*m)-2*a*d*e^3*g)*mz*q*qz-2*S0*a*d*e^3*g^2*qz^2)
             *r+d^2*e^4*g^2*qz^2)
        +((a*m+a*e)*mz*q-S0*a*e*g*qz)*r+d*e^2*g*qz)
      /((2*e*g*m+2*e^2*g)*q*qz*r))
  })
}

NCeq <- function(pars) {
  with(as.list(pars), {
    (d*e)/(m*q*r)
  })
}

Eeq <- function(pars) {
  with(as.list(pars), {
    NE <- pmax(0, NEeq(pars))
    qz*e*NE/mz
  })
}

Seq <- function(pars) {
  with(as.list(pars), {
    E <- Eeq(pars)
    S0*a/(E*g + a)
  })
}

#ODE versions
ECmod <- function(Time, State, Pars) {
  with(as.list(c(State, Pars)), {
    
    dNE <- (r*RE-(e+m))*NE
    dNC <- (r*RC-m)*NC
    dE <- qz*e*NE-mz*E
    dRE <- E*S*g-q*RE*r*NE-d*RE+d*RC
    dRC <- -d*RC+d*RE-(q*RC*r*NC)
    dS <- a*(S0-S)-E*S*g
    
    return(list(c(dNE, dNC, dE, dRE, dRC, dS)))
  })
}
